import preact from 'preact';

/**
 * Removes `-` fron a string and capetalize the letter after
 * example: data-props-hello-world =>  dataPropsHelloWorld
 * Used for props passed from host DOM element
 * @param  {String} str string
 * @return {String} Capetalized string
 */
var camelcasize = function camelcasize(str) {
  return str.replace(/-([a-z])/gi, function (all, letter) {
    return letter.toUpperCase();
  });
};

/**
 * [getExecutedScript internal widget to provide the currently executed script]
 * @param  {document} document [Browser document object]
 * @return {HTMLElement}     [script Element]
 */
var getExecutedScript = function getExecutedScript() {
  return document.currentScript || function () {
    var scripts = document.getElementsByTagName("script");
    return scripts[scripts.length - 1];
  }();
};

/**
 * Get the props from a host element's data attributes
 * @param  {Element} tag The host element
 * @return {Object}  props object to be passed to the component
 */
var collectPropsFromElement = function collectPropsFromElement(element) {
  var attrs = element.attributes;
  var props = {};
  // collect from element
  Object.keys(attrs).forEach(function (key) {
    if (attrs.hasOwnProperty(key)) {
      var dataAttrName = attrs[key].name;
      if (!dataAttrName || typeof dataAttrName !== "string") {
        return false;
      }
      var propName = dataAttrName.split(/(data-props?-)/).pop();
      propName = camelcasize(propName);
      if (dataAttrName !== propName) {
        var propValue = attrs[key].nodeValue;
        props[propName] = propValue;
      }
    }
  });

  //check for child script text/props
  [].forEach.call(element.getElementsByTagName('script'), function (scrp) {
    var propsObj = {};
    if (scrp.hasAttribute('type')) {
      if (scrp.getAttribute('type') !== 'text/props') return;
      try {
        propsObj = JSON.parse(scrp.innerHTML);
      } catch (e) {
        throw new Error(e);
      }
      Object.assign(props, propsObj);
    }
  });

  return props;
};

var getHabitatSelectorFromClient = function getHabitatSelectorFromClient(currentScript) {
  var scriptTagAttrs = currentScript.attributes;
  var selector = null;
  // ceck for another props attached to the tag
  Object.keys(scriptTagAttrs).forEach(function (key) {
    if (scriptTagAttrs.hasOwnProperty(key)) {
      var dataAttrName = scriptTagAttrs[key].name;
      if (dataAttrName === 'data-mount-in') {
        selector = scriptTagAttrs[key].nodeValue;
      }
    }
  });
  return selector;
};

/**
 * Return array of 0 or more elements that will host our widget
 * @param  {id} attrId the data widget id attribute the host should have
 * @param  {document} scope  Docuemnt object or DOM Element as a scope
 * @return {Array}        Array of matching habitats
 */
var widgetDOMHostElements = function widgetDOMHostElements() {
  var selector = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$inline = _ref.inline,
      inline = _ref$inline === undefined ? false : _ref$inline,
      _ref$clientSpecified = _ref.clientSpecified,
      clientSpecified = _ref$clientSpecified === undefined ? false : _ref$clientSpecified,
      _ref$clean = _ref.clean,
      clean = _ref$clean === undefined ? false : _ref$clean;

  var hostNodes = [];
  var currentScript = getExecutedScript();

  if (inline === true) {
    var parentNode = currentScript.parentNode;
    if (clean) node.innerHTML = "";
    hostNodes.push(parentNode);
  }

  if (clientSpecified === true && !selector) {
    // user did not specify where to mount - get it from script tag attributes
    selector = "[data-widget='" + getHabitatSelectorFromClient(currentScript) + "']";
  }
  if (selector) {
    [].forEach.call(document.querySelectorAll(selector), function (queriedTag) {
      if (clean) queriedTag.innerHTML = "";
      hostNodes.push(queriedTag);
    });
  }
  return hostNodes;
};

/**
 * private _render function that will be queued if the DOM is not render
 * and executed immeidatly if DOM is ready
 */
var _render = function _render(widget, hostElements, root) {
  hostElements.forEach(function (elm) {
    var hostNode = elm;
    var props = collectPropsFromElement(elm) || {};
    return preact.render(preact.h(widget, props), hostNode, root);
  });
};

var habitat = function habitat(Widget) {
  // Widget represents the Preact component we need to mount
  var widget = Widget;
  // in case DOM has fired multipled readystate events, redner only once
  var hasRendered = false;
  // preact root render helper
  var root = null;

  var render = function render() {
    var selector = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

    var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        _ref$inline = _ref.inline,
        inline = _ref$inline === undefined ? false : _ref$inline,
        _ref$clean = _ref.clean,
        clean = _ref$clean === undefined ? false : _ref$clean,
        _ref$clientSpecified = _ref.clientSpecified,
        clientSpecified = _ref$clientSpecified === undefined ? false : _ref$clientSpecified;

    var elements = widgetDOMHostElements(selector, { inline: inline, clean: clean, clientSpecified: clientSpecified });
    if (!hasRendered && elements.length > 0) {
      hasRendered = true;
      return _render(widget, elements, root);
    }
    // document is not ready - subscurib to readystatechange event
    document.addEventListener('DOMContentLoaded', function (e) {
      if (!hasRendered && elements.length > 0) {
        var _elements = widgetDOMHostElements(selector, { inline: inline, clean: clean, clientSpecified: clientSpecified });
        hasRendered = true;
        return _render(widget, _elements, root);
      }
    });
  };

  return { render: render };
};

export default habitat;
//# sourceMappingURL=preact-habitat.es.js.map
